// LED

//#include <tinyNeoPixel.h> // neither FastLed.h nor AdaFruit.h work with Attiny
#include <tinyNeoPixel_Static.h>

#define NUMPIXELS 40 

#define BRIGHTNESS 20

#include "millisDelay.h" 

// Which pin on the Arduino is connected to the NeoPixels?
#define CLOCK_LED        0 // it is connected to pin 0 of Attiny1614

byte pixels[NUMPIXELS * 3];
tinyNeoPixel leds = tinyNeoPixel(NUMPIXELS, CLOCK_LED, NEO_GRB, pixels);


// When setting up the NeoPixel library, we tell it how many pixels,
// and which pin to use to send signals. Note that for older NeoPixel
// strips you might need to change the third parameter -- see the
// strandtest example for more information on possible values.
//tinyNeoPixel pixels(NUMPIXELS, CLOCK_LED, NEO_GRB + NEO_KHZ800);

// 15000

#define SEA_DELAYVAL 6000 // Time (in milliseconds) to pause between pixels
#define AIR_DELAYVAL SEA_DELAYVAL/2 // Time (in milliseconds) to pause between pixels


millisDelay seaDelay;
millisDelay airDelay;
millisDelay testDelay;


int mid = 17;

// SEA
int sea_num_leds_on = 0;
int sea_start_led = mid;
bool flood = true;

// AIR
int air_num_leds = 20;
int air_start_left = 7;
int air_start_right = 27;

bool sunset = true;
int red = 255;
int yellow = 255;
int blue = 0;

int max_cycles = 50;
int cycles = 0;

// BORDER
int left_sea_air = sea_start_led - (sea_num_leds_on / 2);
int right_sea_air = sea_start_led + (sea_num_leds_on / 2);

// end LED

// PUMP
#define OUT_1 2
#define OUT_2 3

// SENSOR

//#define sensorPin 1

// Value for storing water level
//int val = 0;

// end SENSOR

#define TEST_LED 10


// the setup function runs once when you press reset or power the board
void setup() {
    delay( 1000 ); // power-up safety delay

    // test LED
    pinMode(TEST_LED, OUTPUT);

     // clock LED
    pinMode(CLOCK_LED, OUTPUT);
    leds.begin(); // INITIALIZE NeoPixel strip object (REQUIRED)    

    leds.setBrightness(BRIGHTNESS);


    //leds.setPixelColor(0, 255, 0, 0); // first LED full RED
    leds.show();

    // sensor
//  // Set D7 as an OUTPUT
//  pinMode(sensorPower, OUTPUT);
  
    // Set to LOW so no power flows through the sensor
    //digitalWrite(sensorPower, LOW);
  
    //Serial.begin(9600);    

    // PUMP
    pinMode(OUT_1, OUTPUT);
    pinMode(OUT_2, OUTPUT);    

    // ANIM
    seaDelay.start(SEA_DELAYVAL); 
    airDelay.start(AIR_DELAYVAL); 

    fillSea(sea_start_led,sea_num_leds_on);
    fillAir(left_sea_air, right_sea_air);


//      testDelay.start(1500);

}

// the loop function runs over and over again forever
void loop() {
  // test LED
  //digitalWrite(TEST_LED, HIGH);   // turn the LED on (HIGH is the voltage level)
  if (testDelay.justFinished()) {
    leds.setBrightness(BRIGHTNESS);
    for(int i=0; i < NUMPIXELS; i++) { 
      leds.setPixelColor(i, leds.Color(200, 200, 100));
    }     
    leds.show(); 
  }


  if (seaDelay.justFinished()) {
    fillSea(sea_start_led,sea_num_leds_on); // from, num

    if (flood) {
      if (sea_num_leds_on > 18) {
        flood = false;
      }
      else {
        sea_start_led -= 1;
        sea_num_leds_on += 2;              
      }
    }
    else {
      if (sea_num_leds_on <= 0 ) {
        flood = true;
      }
      else {
        sea_start_led += 1;
        sea_num_leds_on -= 2;        
      }
 
    }

    // update borders
    left_sea_air = sea_start_led ;//- (sea_num_leds_on / 2);
    right_sea_air = sea_start_led + (sea_num_leds_on / 2);


    seaDelay.repeat(); 
  }  

  if (airDelay.justFinished()) {
        leds.clear();
      fillAir(left_sea_air, right_sea_air);

      airDelay.repeat();
  }

// digitalWrite(OUT_1, LOW);
// digitalWrite(OUT_2, HIGH);  

    // MOTOR
    if (flood) {
      digitalWrite(OUT_1, HIGH);
      digitalWrite(OUT_2, LOW);  
    }
    else {
      digitalWrite(OUT_1, LOW);
      digitalWrite(OUT_2, HIGH);      
    }

  // SENSOR
  //  Serial.print("Water level: ");
  //  int level = analogRead(sensorPin);
  //  Serial.println(level);  
}

void fillAir(int left, int right) {
    for(int i=0; i < left; i++) { 
      leds.setPixelColor(i, leds.Color(red, yellow, blue));
    }  
    for(int i=right; i < NUMPIXELS; i++) { 
      leds.setPixelColor(i, leds.Color(red, yellow, blue));
    }  

    cycles += 1;
    if (cycles > max_cycles)
    {
      cycles = 0;
      sunset = !sunset;
    }

    if (sunset) {
      // turn down yellow
      yellow -= 5;
     // red += 5;

      if (yellow < 0) {
        sunset = false;
      }
    }
    else {
      yellow += 5;
      //red -= 5;
    }
    //leds.show(); 
    
}

void fillSea(int from, int num) {

    int to = from + num;
    //int diff = to - from;
    int mid = from + num/2;

    int mp = 10;
    
    for(int i=from; i< mid; i++) { 
      leds.setPixelColor(i, leds.Color(0, i*mp, 255 - i * mp));
    }  

    for(int i=mid; i<to; i++) { 
      leds.setPixelColor(i, leds.Color(0, i*mp - (mp*8), 255 - (i*mp - (mp*8)) ));
    }  
    
    leds.show();   // Send the updated pixel colors to the hardware.

}
