
//#define TESTING 1

#define LED A1
#define BUTTON A2

static const unsigned int long_delay = 1000; // 1000 ms = 1 sec
static const unsigned int short_delay = 300; // 500 ms = 0.5 sec
static const unsigned int reset_delay = 4000;
static const unsigned int pause_delay = 1000;
static const unsigned int range = 250;

boolean pause = false;

long button_time = 0;     // ts of button press
long play_time   = 0;     // ts of simon says 0 or 1

// NO String! This adds 33k to the program and doesn't even work
//String sequence = "01010010110101010001";//10010110101010001"; // 1 - long light, 0 is short light

// C strings, much better
char sequence[] = "01010010110101010001";

int simon_current = 0;      // index of the character the game is at
int playing_current = 0;    // index of the character simon says
int listening_current = 0;  // index of the character simon hears

int button_state = -1;      // button press = 0, release = 1

int mode = 0;               // 0 is idle, 1 is playing, 2 is listening 

char c;                     // the current character, used both in playing and in listening mode

void setup() {

    pinMode(LED, OUTPUT);
    pinMode(BUTTON, INPUT_PULLUP);

    #ifdef TESTING
        Serial.begin(9600);
        while(!Serial);
    #endif    
}

void loop() {
    button_state = digitalRead(BUTTON); // 0 is down, 1 is up
    
    if (mode == 0) {
        
        // analogWrite can make the LED dim
        analogWrite(LED,2);   
        
        // idle mode
        // wait for a button press to start playing
        if (button_state == 0) {
            mode = 1;
        }
    }
    else if (mode == 1) {
        // playing

        // get the current character from the sequence, either 0 or 1
        if (play_time == 0) {
            c = sequence[playing_current];
        }

        // are we pausing between playing characters? 
        if (pause) {
            // if play time is 0, set it to the current ts
            if (play_time == 0) {
                digitalWrite(LED,LOW);
                play_time = millis();
            }
            else {
                // if we have passed the pause delay time we are done pausing
                long diff = millis() - play_time;
                if (diff > pause_delay) {
                    
                    play_time = 0;
                    digitalWrite(LED,LOW);
                    pause = false;
                }         
            }
        }
        else {
            int delay;
            if (c == '0') {
                // show a short light
                delay = short_delay;
            }
            else {
                // long light
                delay = long_delay;
            }

            if (play_time == 0) {
                play_time = millis();
                digitalWrite(LED,HIGH);
            }
            else {
                long diff = millis() - play_time;
                if (diff > delay) {
                    digitalWrite(LED,LOW);                                   
                    play_time = 0;
                }         
            }

            // play time is 0 again when pause, short or long lights are completed
            if (play_time == 0) {
                if (playing_current == simon_current) {
                    // we have played the complete sequence until now, start listening
                    mode = 2;
                    playing_current = 0;
                    listening_current = 0;
                }
                else {
                    // sequence not yet completed, go to the next character
                    playing_current = playing_current + 1;
                    pause = true;
                }
            }
        }      
    }
    else if (mode == 2){
        // listening, get the character to listen for 
        c = sequence[listening_current];
                
        if (button_time == 0 && button_state == 0) {

            // button was pressed and no ts set yet
            // do it now 
            button_time = millis();

            // light on
            digitalWrite(LED,HIGH);  

        }
        else if (button_time > 0 && button_state == 1 ) {
            // button was released and there was a ts set
            // so we can calculate how long the button was pressed
            long diff = millis() - button_time;
          
            // reset the button ts
            button_time = 0;

            // light off
            digitalWrite(LED,LOW);       
          
            // some housekeeping switches
            boolean correct = false;
            boolean reset   = false;

            // very long press, perform reset
            if( diff > reset_delay ) {
                reset = true;

                playing_current = 0;
                listening_current = 0;
                simon_current = 0;
                button_state = 0;
                pause = false;                
            }
            else if( diff >= (long_delay - range) && diff < long_delay + range) {
                // long press, are we at a long character (1)?
                correct = (c == '1');
            }          
            else if( diff >= (short_delay - range) && diff < short_delay + range) {
                // short press, are we at a short character (0)?
                correct = (c == '0');
            }

            // was there a reset?
            if (reset) {
                mode = 0;
            }
            else if (correct) {
                
                // check if we have completed the sequence
                if (listening_current == simon_current) {
                    
                    // we are done, up simon and start waiting again
                    mode = 1;
                    
                    playing_current = 0;
                    listening_current = 0;
                    // TODO: check for end of Simon :-)
                    simon_current = simon_current + 1;
                    button_state = 0;
                    pause = true;
                    digitalWrite(LED,LOW);     
                }
                else {
                    // not completed sequence, move on to the next character
                    listening_current = listening_current + 1;
                }
            }
            // what if not correct? just wait?
        }
        // else {
        //     analogWrite(LED,10);
        // }
    }
}