// Setup Web Serial using serial.js
const serial = new Serial();
serial.on(SerialEvents.CONNECTION_OPENED, onSerialConnectionOpened);
serial.on(SerialEvents.CONNECTION_CLOSED, onSerialConnectionClosed);
serial.on(SerialEvents.DATA_RECEIVED, onSerialDataReceived);
serial.on(SerialEvents.ERROR_OCCURRED, onSerialErrorOccurred);

let requestSent = false;


function onSerialErrorOccurred(eventSender, error) {
    console.log("onSerialErrorOccurred", error);
    showElement('bt-connect');
}

function onSerialConnectionOpened(eventSender) {
    console.log("onSerialConnectionOpened", eventSender);

    showElement('msg-connected');
    // document.getElementById("cat-image").src = "images/waitcat.gif"

}

function onSerialConnectionClosed(eventSender) {
    console.log("onSerialConnectionClosed", eventSender);
}

function onSerialDataReceived(eventSender, newData) {
    //console.log("onSerialDataReceived", newData);

    if (newData == 0) {
        if (!requestSent) {

            requestSent = true;
            // every time the button is pressed, get me a cat
            document.getElementById("cat-image").src = "images/waitcat.gif"

            console.log("get cat")
            
            /*
  
            this uses
              https://api.thecatapi.com/v1/images/search
  
              it will give a json response like this:
  
              [
                {
                  "breeds": [
                    
                  ],
                  "id": "vKcrwdUwo",
                  "url": "https://cdn2.thecatapi.com/images/vKcrwdUwo.jpg",
                  "width": 876,
                  "height": 657
                }
              ]
            */
            // html request
            var xmlhttp = new XMLHttpRequest();
  
            xmlhttp.onreadystatechange = function() {
                if (this.readyState == 4 && this.status == 200) {
  
                    // we got json back, parse it
                    var myArr = JSON.parse(this.responseText);
  
                    console.log("got cat")
  
                    // get the first element and take property url, use
                    // it to set the src attribute of the image in the html file
                    document.getElementById("cat-image").src = myArr[0].url;
  
                }
            };
  
            // do the request
            xmlhttp.open("GET",  "https://api.thecatapi.com/v1/images/search", true);
            xmlhttp.send();
        }
    }
}

function showElement(id) {
    document.getElementById("bt-connect").style.display = 'none';
    document.getElementById("msg-connected").style.display = 'none';
    document.getElementById("msg-select-port").style.display = 'none';

    document.getElementById(id).style.display = 'flex';
}

async function onConnectButtonClick() {

    if (navigator.serial) {
        if (!serial.isOpen()) {
            await serial.connectAndOpen(null, { baudRate: 115200 });
        } else {
            console.log("The serial connection appears already open");
        }

    } else {
        alert('The Web Serial API does not appear supported on this web browser.');
    }
}

document.getElementById("cat-image").addEventListener("load", event => {
    console.log("cat has loaded");
    requestSent = false;
});

document.getElementById("bt-connect").addEventListener("click",()=>{
    onConnectButtonClick();
    showElement('msg-select-port');

});


showElement('bt-connect');