
const { SerialPort } = require('serialport')
const { ReadlineParser } = require('@serialport/parser-readline')
const WebSocket = require('ws');


// Change the serial port to the port that your board is connected to
const SERIAL_PORT = '/dev/tty.usbmodem21101';

// you can probably leave these as is
const BAUD_RATE = 9600;
const WEBSOCKET_PORT = 1337;

// SERIAL PORT 

// open the serialport
console.log("opening serial port on %s",SERIAL_PORT);

const serialPort = new SerialPort({
  path: SERIAL_PORT,
  baudRate: BAUD_RATE,
})

// error handling (no connection, no port etc)
serialPort.on("error",err => {
  console.log(err);
});

// fires when the serial port opens no problem
serialPort.on("open", data => {
    console.log("-serial port opened succesfully");
});


// this receives and displays raw data
serialPort.on("data", data => {
   //console.log(data);
});


// create a parser to parse the raw data
const parser = new ReadlineParser();
serialPort.pipe( parser );
// whenever we get data from the serial port....
parser.on("data", data => {
        // send it to the clients
        if (websocket) {   
            websocket.send(data);
        }
        else {
            // or pass it to the console if there is no client connected
            console.log(data);
        }

});


// WEBSOCKET SERVER
let websocket = null;

// setup the websocket connection
console.log("setting up socket")
const wss = new WebSocket.Server({ 
  port: WEBSOCKET_PORT 
});

wss.on('error', err => {
  console.log(err);
});

wss.on('listening', (ws) => {
  console.log("-socket connected")
}); 

// connection established
wss.on('connection', (ws) => {
  console.log("-client connected")
  
  ws.on('message', message => {

    // whenever we get a message from any client...
    console.log(message  );

    // pass it to the serial port
    serialPort.write(message );

  })

  // send something to the connected client
  ws.send("hello world");
  
  // save a reference to the websocket, to use in the serial data event handler
  websocket = ws;
});


