// use this when you started client.js, for a version that uses an online server
const WEBSOCKET_ADDRESS = 'wss://socketsbay.com/wss/v2/2/demo/';

// or, when you use local server.js
// set up the ws client, connect on localhost, to the same port that is used in server.js
// WEBSOCKET_ADDRESS = 'ws://localhost:1337';

let socket = new WebSocket(WEBSOCKET_ADDRESS);
let requestSent = false;


socket.onopen = function(e) {

  console.log("connected")

  console.log("[open] Connection established");
  console.log("Sending to server");

  showMessage( '[open] connection established', 'status' );
  document.getElementById("msg-user").style.display = 'flex'
  document.getElementById("bt-reload").style.display = 'none'


  // this is how you send a message to the server (and thus to the serialport)
  // socket.send("hi, I am listening!");
};

// whenever we get a message from the server (= from the serialport)
socket.onmessage = function(event) {
   console.log(`[message] Data received from server: ${event.data}`);
  
   //console.log(document.getElementById("myshape").style.left)
   if (event.data == 0) {
    if (!requestSent) {

        requestSent = true;
        // every time the button is pressed, get me a cat
        document.getElementById("cat-image").src = "images/waitcat.gif"

        console.log("get cat")
        
        /*

        this uses
          https://api.thecatapi.com/v1/images/search

          it will give a json response like this:

          [
            {
              "breeds": [
                
              ],
              "id": "vKcrwdUwo",
              "url": "https://cdn2.thecatapi.com/images/vKcrwdUwo.jpg",
              "width": 876,
              "height": 657
            }
          ]
        */
        // html request
        var xmlhttp = new XMLHttpRequest();

        xmlhttp.onreadystatechange = function() {
            if (this.readyState == 4 && this.status == 200) {

                // we got json back, parse it
                var myArr = JSON.parse(this.responseText);

                // get the first element and take property url, use
                // it to set the src attribute of the image in the html file
                document.getElementById("cat-image").src = myArr[0].url;

            }
        };

        // do the request
        xmlhttp.open("GET",  "https://api.thecatapi.com/v1/images/search", true);
        xmlhttp.send();
    }
}
};


socket.onclose = function(event) {
  if (event.wasClean) {
    showMessage( `[close] Connection closed cleanly, code=${event.code} reason=${event.reason}`, 'status' );

  } 
  else {
    // e.g. server process killed or network down
    // event.code is usually 1006 in this case
    showMessage( '[close] connection died', 'close' );
  }
  document.getElementById("msg-user").style.display = 'none';
  document.getElementById("bt-reload").style.display = 'flex'

};

socket.onerror = function(error) {
    console.log(`[error] ${error.message}`);
    showMessage( `[error] ${error.message}`, error.type );
};


function showMessage(msg, type) {
  //console.log(type)
  switch(type) {
    case 'error':{
      document.getElementById("msg-status").style.backgroundColor = 'crimson';
      break;
    }
    case 'status':{
      document.getElementById("msg-status").style.backgroundColor = 'coral';
      break;
    }
    case 'close':{
      document.getElementById("msg-status").style.backgroundColor = 'lightslategray';
      break;
    }
    default: {
      document.getElementById("msg-status").style.backgroundColor = 'coral';
      break;
    }
  }
  document.getElementById("msg-status").innerText = msg;
}

document.getElementById("cat-image").addEventListener("load", event => {
  requestSent = false;
});


document.getElementById("bt-reload").addEventListener("click",()=>{
    location.reload();
});